#include "Python.h"
#include <fdftk.h>
#include <pthread.h>
#include <stdio.h>
#include <unistd.h>
static PyObject *ErrorObject;      /* local exception */
#define PTHREADS
#ifndef INITIALIZE_FDF_THREADSAFE
#define    INITIALIZE_FDF_THREADSAFE
#endif


static int INIT=0; 

#define onError(message) \
       { PyErr_SetString(ErrorObject, message); return NULL; }

#ifdef PTHREADS
static pthread_mutex_t mutexObj = PTHREAD_MUTEX_INITIALIZER;  
#endif









/*************************************************************/
/*         FDF Interface Begins                              */
/************************************************************/
typedef struct  {
PyObject_HEAD
FDFDoc outFDF;
}fdfobject;
staticforward PyTypeObject Fdftype;

#define is_fdfobject(V) (v(v)->ob_type == &Fdftype)

static PyObject * fdf_FDFCreate( fdfobject *self)
{
  FDFErc err;
  err=FDFCreate(&(self->outFDF));
  if(err != FDFErcOK )
    onError("FDFCreate Failed");
   
  Py_INCREF(Py_None); 
  return Py_None;
}
static PyObject * fdf_FDFSetStatus(fdfobject *self,
    PyObject    *args)
{
  PyObject *pstr;
 
  FDFErc err;
  if (!PyArg_ParseTuple(args, "s",&pstr))     /* convert args: Python->C */       
       onError("Error parsing args for FDFSetStatus");
    
        
#ifdef TEST
  printf("status is %s \n",(const char *)pstr);
#endif
  err = FDFSetStatus(self->outFDF,(const char *)pstr);
  if( err !=FDFErcOK )
    onError("Error setting status");
  Py_INCREF(Py_None); 
  return Py_None;
}
static PyObject * fdf_FDFSetValue(fdfobject * self,
    PyObject    *args)
{
 
  PyObject* ar,*nstr,*vstr;
 if (!PyArg_ParseTuple(args, "ss",&nstr,&vstr))     /* convert args: Python->C */
        return NULL; 
#ifdef TEST
 printf("var is %s ,val is %s \n" ,(const char *)nstr, (const char *)vstr);
#endif
 FDFSetValue(self->outFDF, (const char *)nstr, (const char *)vstr, false); 
 Py_INCREF(Py_None); 
 return Py_None;
}
static PyObject * FDFGetBuffer(fdfobject *self,
    PyObject    *args)
{
  PyObject *pstr;
  char * t;
  
  if (!PyArg_ParseTuple(args, "s",&pstr))     /* convert args: Python->C */
        return NULL; 
#ifdef TEST
  printf("The file is %s \n",pstr); 
#endif
  FDFSave(self->outFDF, (const char *)pstr); 
 

 /*FDFFinalize()*/

 Py_INCREF(Py_None); 
 return Py_None;

}
static PyObject * fdf_FDFClose(fdfobject *self,
			     PyObject    *args)
{
   
  
   FDFClose(self->outFDF);
   Py_INCREF(Py_None); 
   return Py_None;
}

static struct PyMethodDef fdf_methods[] = {
  {"FDFCreate",fdf_FDFCreate,0},
  {"FDFSetStatus",fdf_FDFSetStatus,1},
  {"FDFSetValue",fdf_FDFSetValue,2},
  {"MyFDFGetBuffer",FDFGetBuffer,1},
  {"FDFClose",fdf_FDFClose,0},
  {NULL,NULL}
};

static fdfobject* newfdfobject()
{
  fdfobject *self;
  self = PyObject_NEW(fdfobject,&Fdftype);
  if(self == NULL)
    return NULL;

  return self;
}
static void fdf_dealloc(fdfobject* self)
{
  PyMem_DEL(self);

}
static PyObject *
fdf_getattr(self, name)           /* on "instance.attr" reference  */
    fdfobject *self;              /* make a bound-method or member */
    char *name; 
{   
 
        return Py_FindMethod(fdf_methods, (PyObject *)self, name);
}
static PyTypeObject Fdftype  = {      /* main python type-descriptor */
  /* type header */                    /* shared by all instances */
      PyObject_HEAD_INIT(&PyType_Type)         
      0,                               /* ob_size */
      "fdf",                         /* tp_name */
      sizeof(fdfobject),             /* tp_basicsize */
      0,                               /* tp_itemsize */
  /* standard methods */
      (destructor)  fdf_dealloc,   /* tp_dealloc  ref-count==0  */
       (printfunc)  0,     /* tp_print    "print x"     */
      (getattrfunc) fdf_getattr,   /* tp_getattr  "x.attr"      */
      (setattrfunc) 0,               /* tp_setattr  "x.attr=v"    */
      (cmpfunc)     0,   /* tp_compare  "x > y"       */
      (reprfunc)    0,               /* tp_repr     `x`, print x  */

  /* type categories */
      0,                             /* tp_as_number   +,-,*,/,%,&,>>,pow...*/
      0,            /* tp_as_sequence +,[i],[i:j],len, ...*/
      0,                             /* tp_as_mapping  [key], len, ...*/

  /* more methods */
      (hashfunc)   0,                /* tp_hash    "dict[x]" */
      (binaryfunc) 0,                /* tp_call    "x()"     */
      (reprfunc)   0,                /* tp_str     "str(x)"  */
};
static PyObject *
fdftype_new(self, args)                 /* on "x = fdftype.FDF()" */
    PyObject *self;                       /* self not used */
    PyObject *args;                       /* constructor args */
{
    if (!PyArg_ParseTuple(args, ""))      /* Module-method function */
        return NULL;
    return (PyObject *)newfdfobject();  /* make a new type-instance object */
}                                         /* the hook from module to type... */

static struct PyMethodDef fdftype_methods[] = {
    {"FDF",  fdftype_new,  1},             /* one function: make an FDF type */ 
    {NULL,     NULL}                           /* end marker, for initmodule */
};

/**************************************************
intialize FDF
 **************************************************/
void initfdftype()
{
   PyObject *m, *d;
  if(!INIT)
    {
      FDFErc err;
      INIT=1;
      
#ifdef PTHREADS
      
    pthread_mutex_init( &mutexObj, NULL); 
    err = FDFRegisterThreadsafeCallbacks( pthread_mutex_lock,pthread_mutex_unlock,pthread_mutex_destroy,
				&mutexObj);
    if( err != FDFErcOK)
      Py_FatalError("FDFRegisterThreadSafeCallBacks failed");
#endif
    if(FDFInitialize() != FDFErcOK )
      Py_FatalError("FDFinitialize failed");
    }
    m = Py_InitModule("fdftype", fdftype_methods);   /* make the module, */
    d = PyModule_GetDict(m);                             /* with 'FDF' func */
    ErrorObject = Py_BuildValue("s", "FDF.error");
    PyDict_SetItemString(d, "error", ErrorObject);       /* export exception */
    if (PyErr_Occurred())
        Py_FatalError("can't initialize module PyFDF");
   
}
